@@ -97,10 +97,14 @@ def test_machine_from_config(default_globals: dict):
           target_tag_list: [GRA-TE -123-456]
           train_start_date: 2018-01-01T09:00:30Z
           train_end_date: 2018-01-02T09:00:30Z
+          filter_periods:
+            filter_method: "median"
+            n_iqr: 1
+            window: 72
         model:
           sklearn.pipeline.Pipeline:
             steps:
-              - sklearn.preprocessing.data.MinMaxScaler
+              - sklearn.preprocessing.MinMaxScaler
               - gordo.machine.model.models.KerasAutoEncoder:
                   kind: feedforward_hourglass
         evaluation:
@@ -136,9 +140,11 @@ def test_machine_from_config(default_globals: dict):
                 "type": "DataLakeProvider",
             },
             "default_asset": None,
+            "filter_periods": {"filter_method": "median", "window": 72, "n_iqr": 1},
             "high_threshold": 50000,
             "interpolation_limit": "8H",
             "interpolation_method": "linear_interpolation",
+            "known_filter_periods": [],
             "low_threshold": -1000,
             "n_samples_threshold": 0,
             "resolution": "10T",
@@ -188,7 +194,7 @@ def test_machine_from_config(default_globals: dict):
         "model": {
             "sklearn.pipeline.Pipeline": {
                 "steps": [
-                    "sklearn.preprocessing.data.MinMaxScaler",
+                    "sklearn.preprocessing.MinMaxScaler",
                     {
                         "gordo.machine.model.models.KerasAutoEncoder": {
                             "kind": "feedforward_hourglass"
@@ -228,7 +234,7 @@ def test_invalid_model(default_globals: dict):
         model:
           sklearn.pipeline.Pipeline:
             step:
-              - sklearn.preprocessing.data.MinMaxScaler
+              - sklearn.preprocessing.MinMaxScaler
               - gordo.machine.model.models.KerasAutoEncoder:
                   kind: feedforward_hourglass
         evaluation:
